# Summaries

The angular analyzer, like the Dart analyzer, uses summaries to cache results
and speed up analysis.

## Types of summaries

There are two main summaries, a "linked" and "unlinked" summary.

An unlinked summary is a summarization of the syntactic model -- see
`lib/src/model/syntactic/README.md`. We store these in `$HASH.ng.unlinked`. Note
that we store different unlinked summary formats for HTML vs Dart.

A linked summary is the errors produced during a full analysis. It does not
contain everything required for autocompletion and navigation. In order to
do these things we must analyze from scratch. This is a good thing, because it
makes our summaries faster, and no more than one file needs autocompletion or
navigation data at a time so it is O(1). We store linked summaries in
`$HASH.ng.linked`.

## Caching

The hashes are generated by [FileHasher]. If functionality is added to the
plugin or the summary format is changed, in a non-breaking way, the salt in that
class must be changed.

## Generating the code

The summary itself is generated from `idl.dart`, which defines the interface.
The generator is in the SDK repo `pkg/analyzer/tool/summary/generate.dart`.

## Useful to know

Summaries are serialized to flatbuffers, which have good performance
characteristics for lazy linking etc.

The analysis server spawns an isolate that removes unused summary files, so we
don't have to clean them up.

If users have bad results in their cache, they should run:

```
rm -r ~/.dartServer/.analysis-driver/*.ng*
```
